import axios, { AxiosResponse } from "axios";
import { fileTypeFromBuffer } from "file-type";

import { fsWriteToFile } from "./fsWrite.js";
import { bytesToMb } from "@/convert";

type DownloadFileOptions = {
  url: string;
  fileName?: string;
  directoryPath: string;

  limit?: {
    minBytes?: number;
    maxBytes?: number;
  };
};

interface DownloadFile {
  path: string;
  size: number;
  sizeInMB: string;
  fileType: string;
  fileTypeExt: string;
  fileName: string;
  url: string;
  createdAt: Date;
}

const downloadFileFromUrl = async ({
  url,
  fileName,
  directoryPath,
  limit,
}: DownloadFileOptions): Promise<DownloadFile> => {
  if (!url || !directoryPath) {
    throw new Error("Url and DirectoryPath are required");
  }

  const fileLimit: { minBytes: number; maxBytes: number } = {
    minBytes: 0,
    maxBytes: 100 * 1024 * 1024, // 100 MB
    ...limit,
  };

  const { data, headers } = await axios.get(url, {
    responseType: "arraybuffer",
    maxBodyLength: fileLimit.maxBytes,
  });
  const contentLength: number = parseInt(headers["content-length"] ?? "0", 10);
  if (
    isNaN(contentLength) ||
    contentLength <= 0 ||
    contentLength > fileLimit.maxBytes ||
    contentLength < fileLimit.minBytes
  ) {
    throw new Error(
      `Media size is invalid. Size should be between ${
        fileLimit.maxBytes
      } and ${fileLimit.minBytes} bytes or between ${bytesToMb(
        fileLimit.maxBytes
      )} and ${bytesToMb(fileLimit.minBytes)}`
    );
  }
  const fileType = await fileTypeFromBuffer(data);
  if (!fileType || !isValidMediaType(fileType.mime)) {
    throw new Error("Invalid media type. Should be image, video, pdf or doc");
  }

  const fileNameString: string =
    fileName ?? `${Date.now()}-${Math.random()}.${fileType.ext}`;
  const filePath: string = await fsWriteToFile(
    directoryPath,
    data,
    fileNameString
  );

  const fileData: DownloadFile = {
    path: filePath,
    size: contentLength,
    sizeInMB: `${(contentLength / 1024 / 1024).toFixed(2)} MB`,
    fileType: fileType.mime,
    fileTypeExt: fileType.ext,
    fileName: fileNameString,
    url,
    createdAt: new Date(),
  };

  return fileData;
};

/**
 * Check if the provided media type is valid.
 *
 * @param {string} mediaType - the media type to be checked
 * @return {boolean} true if the media type is valid, false otherwise
 */
const isValidMediaType = (mediaType: string): boolean => {
  return (
    mediaType.startsWith("image/") ||
    mediaType.startsWith("video/") ||
    mediaType.startsWith("application/pdf") ||
    mediaType.startsWith("application/msword") ||
    mediaType.startsWith(
      "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
    )
  );
};

export { downloadFileFromUrl };
