import fs, { writeJson as writeJsonFile } from "fs-extra";

type JSONTypes = Object | Array<any> | Object[] | Array<Object> | string;

/**
 * Reads a JSON file from the given file path and returns its content as a parsed JSON object.
 *
 * @param {string} filePath - The path of the JSON file to be read.
 * @return {Promise<JSONTypes>} A promise that resolves to the parsed JSON object.
 * @throws {Error} If there was an error reading the file or parsing its content.
 */
const readJSONFile = async (filePath: string): Promise<JSONTypes> => {
  const fileContent = await fs.readFile(filePath, "utf8");
  try {
    return JSON.parse(fileContent);
  } catch (error) {
    throw error;
  }
};

/**
 * Write JSON data to a file.
 *
 * @param {string} filePath - the path to the file
 * @param {JSONTypes} jsonData - the JSON data to be written
 * @param {fs.JsonWriteOptions} [options] - the options for writing JSON
 * @return {Promise<boolean>} whether the JSON data was successfully written to the file
 */
const writeJsonToFile = async (
  filePath: string,
  jsonData: JSONTypes,
  options?: fs.JsonWriteOptions
): Promise<boolean> => {
  if (typeof filePath !== "string" || filePath.length === 0) {
    throw new Error("Invalid filePath");
  }

  if (jsonData === null || jsonData === undefined) {
    throw new Error("Invalid jsonData");
  }

  let data: JSONTypes;
  if (typeof jsonData === "string") {
    try {
      data = JSON.parse(jsonData);
    } catch (err: unknown) {
      throw new Error("Invalid jsonData");
    }
  } else {
    data = jsonData;
  }

  try {
    await writeJsonFile(filePath, data, options);
    return true;
  } catch (error: unknown) {
    throw new Error(
      `Failed to write JSON file: ${filePath}, error: ${
        (error as Error).message
      }`
    );
  }
};

export { readJSONFile as fsReadJSONFile, writeJsonToFile as fsWriteJsonToFile };
