import { writeFile, ensureDir } from "fs-extra";

/**
 * Writes the given content to a file in the specified directory with the given file name.
 *
 * @example
 * const filePath = await writeToFile('/path/to/directory', 'content', 'filename.txt');
 * console.log(filePath);
 * // '/path/to/directory/filename.txt'
 *
 * If the directory does not exist, it will be created.
 * If the file already exists, it will be overwritten.
 * If the directory or the file cannot be created, an error will be thrown.
 *
 * @param {string} directory - The directory where the file will be written.
 * @param {string | Buffer | Uint8Array} content - The content to be written to the file.
 * @param {string} fileName - The name of the file.
 * @return {Promise<string>} - A promise that resolves to the file path where the content was written.
 */
const writeToFile = async (
  directory: string,
  content: string | Buffer | Uint8Array,
  fileName: string
): Promise<string> => {
  await ensureDir(directory);
  const filePath = `${directory}/${fileName}`;
  await writeFile(filePath, content);
  return filePath;
};

export { writeToFile as fsWriteToFile };
